#ifndef Mat_INCLUDED
#define Mat_INCLUDED

#pragma warning( disable : 4244 )

/*

  interface:

  struct Mat {
    Pnt transform(const Pnt& p) const;
    Vec transform(const Vec& p) const;
  };
  Mat invert(const Mat&);
  Mat operator*(const Mat&, const mat&);

*/



struct Mat {

  float m[4][4];

  Mat() {
    m[0][0] = 1.0;  m[0][1] = 0.0;  m[0][2] = 0.0;  m[0][3] = 0.0;
    m[1][0] = 0.0;  m[1][1] = 1.0;  m[1][2] = 0.0;  m[1][3] = 0.0;
    m[2][0] = 0.0;  m[2][1] = 0.0;  m[2][2] = 1.0;  m[2][3] = 0.0;
    m[3][0] = 0.0;  m[3][1] = 0.0;  m[3][2] = 0.0;  m[3][3] = 1.0;
  }

  Mat(float a00, float a01, float a02, float a03,
      float a10, float a11, float a12, float a13,
      float a20, float a21, float a22, float a23,
      float a30, float a31, float a32, float a33) {

    m[0][0] = a00;  m[0][1] = a01;  m[0][2] = a02;  m[0][3] = a03;
    m[1][0] = a10;  m[1][1] = a11;  m[1][2] = a12;  m[1][3] = a13;
    m[2][0] = a20;  m[2][1] = a21;  m[2][2] = a22;  m[2][3] = a23;
    m[3][0] = a30;  m[3][1] = a31;  m[3][2] = a32;  m[3][3] = a33;
  }

};


inline Mat invert(const Mat& from) {

  float wtemp[4][8];
  float m0,m1,m2,m3,s;
  float *r0,*r1,*r2,*r3, *rtemp;

  r0 = wtemp[0];
  r1 = wtemp[1];
  r2 = wtemp[2];
  r3 = wtemp[3];
  r0[0] = from.m[0][0];             /* build up [A][I]      */
  r0[1] = from.m[0][1];
  r0[2] = from.m[0][2];
  r0[3] = from.m[0][3];
  r0[4] = 1.0;
  r0[5] = 0.0;
  r0[6] = 0.0;
  r0[7] = 0.0;
  r1[0] = from.m[1][0];
  r1[1] = from.m[1][1];
  r1[2] = from.m[1][2];
  r1[3] = from.m[1][3];
  r1[4] = 0.0;
  r1[5] = 1.0;
  r1[6] = 0.0;
  r1[7] = 0.0;
  r2[0] = from.m[2][0];
  r2[1] = from.m[2][1];
  r2[2] = from.m[2][2];
  r2[3] = from.m[2][3];
  r2[4] = 0.0;
  r2[5] = 0.0;
  r2[6] = 1.0;
  r2[7] = 0.0;
  r3[0] = from.m[3][0];
  r3[1] = from.m[3][1];
  r3[2] = from.m[3][2];
  r3[3] = from.m[3][3];
  r3[4] = 0.0;
  r3[5] = 0.0;
  r3[6] = 0.0;
  r3[7] = 1.0;
  
  if (r0[0] == 0.0) {             /* swap rows if needed          */
    if (r1[0] == 0.0) {
      if (r2[0] == 0.0) {
	if (r3[0] == 0.0) return Mat(); // singular
	rtemp = r0;
	r0 = r3;
	r3 = rtemp;
      } else {
	rtemp = r0;
	r0 = r2;
	r2 = rtemp;
      }
    } else {
      rtemp = r0;
      r0 = r1;
      r1 = rtemp;
    }
  }
  m1 = r1[0]/r0[0];               /* eliminate first variable     */
  m2 = r2[0]/r0[0];
  m3 = r3[0]/r0[0];
  s = r0[1];
  r1[1] = r1[1] - m1 * s;
  r2[1] = r2[1] - m2 * s;
  r3[1] = r3[1] - m3 * s;
  s = r0[2];
  r1[2] = r1[2] - m1 * s;
  r2[2] = r2[2] - m2 * s;
  r3[2] = r3[2] - m3 * s;
  s = r0[3];
  r1[3] = r1[3] - m1 * s;
  r2[3] = r2[3] - m2 * s;
  r3[3] = r3[3] - m3 * s;
  s = r0[4];
  if (s != 0.0) {
    r1[4] = r1[4] - m1 * s;
    r2[4] = r2[4] - m2 * s;
    r3[4] = r3[4] - m3 * s;
  }
  s = r0[5];
  if (s != 0.0) {
    r1[5] = r1[5] - m1 * s;
    r2[5] = r2[5] - m2 * s;
    r3[5] = r3[5] - m3 * s;
  }
  s = r0[6];
  if (s != 0.0) {
    r1[6] = r1[6] - m1 * s;
    r2[6] = r2[6] - m2 * s;
    r3[6] = r3[6] - m3 * s;
  }
  s = r0[7];
  if (s != 0.0) {
    r1[7] = r1[7] - m1 * s;
    r2[7] = r2[7] - m2 * s;
    r3[7] = r3[7] - m3 * s;
  }

  if (r1[1] == 0.0) {             /* swap rows if needed          */
    if (r2[1] == 0.0) {
      if (r3[1] == 0.0) return Mat(); // singular
      rtemp = r1;
      r1 = r3;
      r3 = rtemp;
    } else {
      rtemp = r1;
      r1 = r2;
      r2 = rtemp;
    }
  }
  m2 = r2[1]/r1[1];               /* eliminate second variable    */
  m3 = r3[1]/r1[1];
  r2[2] = r2[2] - m2 * r1[2];
  r3[2] = r3[2] - m3 * r1[2];
  r3[3] = r3[3] - m3 * r1[3];
  r2[3] = r2[3] - m2 * r1[3];
  s = r1[4];
  if (s != 0.0) {
    r2[4] = r2[4] - m2 * s;
    r3[4] = r3[4] - m3 * s;
  }
  s = r1[5];
  if (s != 0.0) {
    r2[5] = r2[5] - m2 * s;
    r3[5] = r3[5] - m3 * s;
  }
  s = r1[6];
  if (s != 0.0) {
    r2[6] = r2[6] - m2 * s;
    r3[6] = r3[6] - m3 * s;
  }
  s = r1[7];
  if (s != 0.0) {
    r2[7] = r2[7] - m2 * s;
    r3[7] = r3[7] - m3 * s;
  }

  if (r2[2] == 0.0) {             /* swap last 2 rows if needed   */
    if (r3[2] == 0.0) return Mat(); // singular
    rtemp = r2;
    r2 = r3;
    r3 = rtemp;
  }
  m3 = r3[2]/r2[2];               /* eliminate third variable     */
  r3[3] = r3[3] - m3 * r2[3];
  r3[4] = r3[4] - m3 * r2[4];
  r3[5] = r3[5] - m3 * r2[5];
  r3[6] = r3[6] - m3 * r2[6];
  r3[7] = r3[7] - m3 * r2[7];

  if (r3[3] == 0.0) return Mat(); // singular
  s = 1.0/r3[3];          /* now back substitute row 3    */
  r3[4] = r3[4] * s;
  r3[5] = r3[5] * s;
  r3[6] = r3[6] * s;
  r3[7] = r3[7] * s;

  m2 = r2[3];                     /* now back substitute row 2    */
  s = 1.0/r2[2];
  r2[4] = s * (r2[4] - r3[4] * m2);
  r2[5] = s * (r2[5] - r3[5] * m2);
  r2[6] = s * (r2[6] - r3[6] * m2);
  r2[7] = s * (r2[7] - r3[7] * m2);
  m1 = r1[3];
  r1[4] = (r1[4] - r3[4] * m1);
  r1[5] = (r1[5] - r3[5] * m1);
  r1[6] = (r1[6] - r3[6] * m1);
  r1[7] = (r1[7] - r3[7] * m1);
  m0 = r0[3];
  r0[4] = (r0[4] - r3[4] * m0);
  r0[5] = (r0[5] - r3[5] * m0);
  r0[6] = (r0[6] - r3[6] * m0);
  r0[7] = (r0[7] - r3[7] * m0);

  m1 = r1[2];                     /* now back substitute row 1    */
  s = 1.0/r1[1];
  r1[4] = s * (r1[4] - r2[4] * m1);
  r1[5] = s * (r1[5] - r2[5] * m1);
  r1[6] = s * (r1[6] - r2[6] * m1);
  r1[7] = s * (r1[7] - r2[7] * m1);
  m0 = r0[2];
  r0[4] = (r0[4] - r2[4] * m0);
  r0[5] = (r0[5] - r2[5] * m0);
  r0[6] = (r0[6] - r2[6] * m0);
  r0[7] = (r0[7] - r2[7] * m0);

  m0 = r0[1];                     /* now back substitute row 0    */
  s = 1.0/r0[0];
  r0[4] = s * (r0[4] - r1[4] * m0);
  r0[5] = s * (r0[5] - r1[5] * m0);
  r0[6] = s * (r0[6] - r1[6] * m0);
  r0[7] = s * (r0[7] - r1[7] * m0);

  return Mat(r0[4], r0[5], r0[6], r0[7],
	     r1[4], r1[5], r1[6], r1[7],
	     r2[4], r2[5], r2[6], r2[7],
	     r3[4], r3[5], r3[6], r3[7]);
}


inline Mat operator*(const Mat& a, const Mat& b) {

  float result[4][4];

  for (int i=0; i<4; ++i) {
    for (int j=0; j<4; ++j) {
      float s = 0;
      for (int k=0; k<4; ++k) {
	s += a.m[i][k] * b.m[k][j];
      }
      result[i][j] = s;
    }
  }

  return *reinterpret_cast<Mat*>(&result[0][0]);
}


#endif
